@@ -1,11 +1,11 @@
 /*
- * misc.c -- description
- * Part of the netcat project
+ * misc.c -- contains generic needed routines
+ * Part of the GNU netcat project
  *
- * Author: Johnny Mnemonic <johnny@themnemonic.org>
- * Copyright (c) 2002 by Johnny Mnemonic
+ * Author: Giovanni Giacobbi <johnny@themnemonic.org>
+ * Copyright (C) 2002  Giovanni Giacobbi
  *
- * $Id: misc.c,v 1.13 2002/05/01 16:04:45 themnemonic Exp $
+ * $Id: misc.c,v 1.20 2002/05/06 20:37:44 themnemonic Exp $
  */
 
 /***************************************************************************
@@ -30,10 +30,11 @@
 
 /* Hexdump `datalen' bytes starting at `data' to the file pointed to by `stream'.
    If the given block generates a partial line it's rounded up with blank spaces.
-   This function was written by Johnny Mnemonic for the netcat project,
+   This function was written by Giovanni Giacobbi for The GNU Netcat project,
    credits must be given for any use of this code outside this project */
 
-int netcat_fhexdump(FILE *stream, const unsigned char *data, size_t datalen)
+int netcat_fhexdump(FILE *stream, char c, const unsigned char *data,
+		    size_t datalen)
 {
   size_t pos;
   char buf[80], *ascii_dump, *p = NULL;
@@ -57,7 +58,7 @@ int netcat_fhexdump(FILE *stream, const unsigned char *data, size_t datalen)
 #ifndef USE_OLD_HEXDUMP
       p += sprintf(p, "%08X  ", (unsigned int) pos);
 #else
-      p += sprintf(p, "? %08X ", (unsigned int) pos);
+      p += sprintf(p, "%c %08X ", c, (unsigned int) pos);
 #endif
     }
 
@@ -113,17 +114,60 @@ int netcat_fhexdump(FILE *stream, const unsigned char *data, size_t datalen)
 
 /* ... */
 
-void debug_output(bool wrap, const char *fmt, ...)
+void ncprint(int type, const char *fmt, ...)
 {
+  int flags = type & 0xFF;
+  char buf[1024], newline = '\n';
+  FILE *fstream = NULL;
   va_list args;
 
+  type &= ~0xFF;
+
+#ifndef DEBUG
+  /* return if this requires some verbosity levels and we haven't got it */
+  if ((flags & NCPRINT_VERB2) && (opt_verbose < 2))
+    return;
+
+  if ((flags & NCPRINT_VERB1) && (opt_verbose < 1))
+    return;
+#endif
+
+  /* known flags */
+  if (flags & NCPRINT_STDERR)
+    fstream = stderr;
+  else if (flags & NCPRINT_STDOUT)
+    fstream = stdout;
+  else if (flags & NCPRINT_NONEWLINE)
+    newline = '\0';
+
+  /* from now on, we are sure that we need the string formatted */
   va_start(args, fmt);
-  if (wrap)
-    printf("(debug) ");
-  vprintf(fmt, args);
-  if (wrap)
-    printf("\n");
-  va_end(args);
+  vsnprintf(buf, sizeof(buf), fmt, args);
+
+  switch (type) {
+  case NCPRINT_NORMAL:
+    fprintf((fstream ? fstream : stdout), "%s%c", buf, newline);
+    break;
+#ifdef DEBUG
+  case NCPRINT_DEBUG:
+    fprintf((fstream ? fstream : stdout), "(debug) %s%c", buf, newline);
+    break;
+#endif
+  case NCPRINT_ERROR:
+    fprintf((fstream ? fstream : stderr), "%s %s%c", _("Error:"), buf, newline);
+    break;
+  case NCPRINT_WARNING:
+    fprintf((fstream ? fstream : stderr), "%s %s%c", _("Warning:"), buf, newline);
+    break;
+  }
+  /* discard unknown types */
+
+  /* post-output effects flags */
+  if (flags & NCPRINT_DELAY)
+    usleep(NCPRINT_WAITTIME);
+
+  if (flags & NCPRINT_EXIT)
+    exit(EXIT_FAILURE);
 }
 
 /* ... */
@@ -134,8 +178,10 @@ char *netcat_string_split(char **buf)
 
   if (!buf)
     return *buf = "";
-  for (o = *buf; isspace(*o); o++);	/* split all initial spaces */
-  for (r = o; *o && !isspace(*o); o++);	/* save the pointer and move to the next token */
+  /* skip all initial spaces */
+  for (o = *buf; isspace((int)*o); o++);
+  /* save the pointer and move to the next token */
+  for (r = o; *o && !isspace((int)*o); o++);
   if (*o)
     *o++ = 0;
   *buf = o;
@@ -144,7 +190,7 @@ char *netcat_string_split(char **buf)
 
 /* construct an argv, and hand anything left over to readwrite(). */
 
-void netcat_commandline(int *argc, char ***argv)
+void netcat_commandline_read(int *argc, char ***argv)
 {
   int my_argc = 1;
   char **my_argv = *argv;
@@ -189,13 +235,14 @@ void netcat_printhelp(char *argv0)
   printf(_("listen for inbound:    %s -l -p port [options] [hostname] [port]\n"), argv0);
   printf("\n");
   printf(_("Mandatory arguments to long options are mandatory for short options too.\n"));
-  /* "  -e, --exec=PROGRAM         program to exec after connect [dangerous!!]\n" */
   printf(_("Options:\n"
+"  -e, --exec=PROGRAM         program to exec after connect\n"
 "  -g, --gateway=LIST         source-routing hop point[s], up to 8\n"
 "  -G, --pointer=NUM          source-routing pointer: 4, 8, 12, ...\n"
 "  -h, --help                 display this help and exit\n"
 "  -i, --interval=SECS        delay interval for lines sent, ports scanned\n"
 "  -l, --listen               listen mode, for inbound connects\n"
+"  -L, --tunnel               forward local port to remote address\n"
 "  -n, --dont-resolve         numeric-only IP addresses, no DNS\n"
 "  -o, --output=FILE          output hexdump traffic to FILE (implies -x)\n"
 "  -p, --local-port=NUM       local port number\n"
@@ -219,11 +266,11 @@ void netcat_printhelp(char *argv0)
 void netcat_printversion(void)
 {
   printf("netcat (The GNU Netcat) %s\n", VERSION);
-  printf(_("Copyright (c) 2002 Johnny Mnemonic\n\n"
+  printf(_("Copyright (C) 2002  Giovanni Giacobbi\n\n"
 "This program comes with NO WARRANTY, to the extent permitted by law.\n"
 "You may redistribute copies of this program under the terms of\n"
 "the GNU General Public License.\n"
 "For more information about these matters, see the file named COPYING.\n\n"
 "Original idea and design by Avian Research <hobbit@avian.org>,\n"
-"Written by Johnny Mnemonic <johnny@themnemonic.org>.\n"));
+"Written by Giovanni Giacobbi <johnny@themnemonic.org>.\n"));
 }
